use std::cmp::max;
use std::str::Chars;

use crate::matrix::Matrix;

pub fn diff(a: &str, b: &str) {
    let m = build_matrix(a, b);
    print_diff(m, a, b, a.len() - 1, b.len() - 1);
}

fn build_matrix(a: &str, b: &str) -> Matrix<i32> {
    let mut m = Matrix::new(a.len(), b.len(), 0i32);

    let mut i = 0;
    let mut j = 0;

    for x in a.chars() {
        for y in b.chars() {
            let v = if x == y {
                if i == 0 || j == 0 {
                    0
                } else {
                    m.get(i - 1, j - 1).unwrap() + 1
                }
            } else {
                max(
                    if j == 0 { 0 } else { *m.get(i, j - 1).unwrap() },
                    if i == 0 { 0 } else { *m.get(i - 1, j).unwrap() })
            };

            m.set(v, i, j).unwrap();

            j += 1;
        }

        i += 1;
        j = 0;
    }

    m
}

fn print_diff(m: Matrix<i32>, a: &str, b: &str, i: usize, j: usize) {
    let char_i = a.chars().nth(i).unwrap();
    let char_j = b.chars().nth(j).unwrap();

    println!("A: {char_i}, B: {char_j}");

    if char_i == char_j {
        println!("  {char_i}");
        return;
    }

    if i > 0 {
        let up = m.get(i - 1, j).unwrap();

        if j > 0 && m.get(i, j - 1).unwrap() >= up {
            println!("+ {char_i}");
            return;
        }
    }

    println!("- {char_i}");
}

// fn print_diff(m: Matrix<i32>, a: &str, b: &str, i: usize, j: usize) {
//     // if i < 0 && j < 0 {
//     //     println!();
//     //     return;
//     // }
//
//     let char_i = a.chars().nth(i).unwrap();
//     let char_j = b.chars().nth(j).unwrap();
//
//     // if i < 0 {
//     //     print_diff(m, a, b, i, j - 1);
//     //     println!("+ {char_i}");
//     //     return;
//     // }
//     //
//     // if j < 0 {
//     //     print_diff(m, a, b, i - 1, j);
//     //     println!("- {char_j}");
//     //     return;
//     // }
//
//     if char_i == char_j {
//         if i > 0 && j > 0 {
//             print_diff(m, a, b, i - 1, j - 1);
//         }
//
//         println!("  {char_i}");
//         return;
//     }
//
//     if i > 0 && j > 0 && m.get(i, j - 1).unwrap() >= m.get(i - 1, j).unwrap() {
//         print_diff(m, a, b, i, j - 1);
//         println!("+ {char_j}");
//         return;
//     }
//
//     if i > 0 {
//         print_diff(m, a, b, i - 1, j);
//     }
//
//     println!("- {char_i}");
// }
